<?php

namespace Slovocast\Infrastructure\Api\Database;

use Exception;

interface ConnectionPoolInterface
{
    public function getTotalIdleConnections(): int;
    public function getTotalActiveConnections(): int;

    /**
     * If there is at least one idle connection waiting to be used.
     *
     * @return bool
     */
    public function hasIdleConnection(): bool;
    public function setWaitTimeout(int $seconds): void;
    public function getWaitTimeout(): int;
    public function getConnectionLimit(): int;

    /**
     * This method will grab a connection from the Idle list, push it into the Active list and returns that same
     * connection to the caller. If there is no active connections, the method will wait based on the `waitTimeout`
     * value (in seconds) and throw an exception when the timeout has been reached.
     *
     * @throws Exception When the Wait Timeout is surpassed waiting for an idle connection
     * @return PooledConnectionInterface
     */
    public function getConnection(): PooledConnectionInterface;

    /**
     * This method will return a connection from the Active list into the Idle list.
     *
     * @param PooledConnectionInterface $connection
     * @return void
     */
    public function releaseConnection(PooledConnectionInterface $connection): void;


}
